@@ -210,7 +210,7 @@ class OmniverseIsaacGymWrapper(Wrapper):
         """Omniverse Isaac Gym environment wrapper
 
         :param env: The environment to wrap
-        :type env: Any supported Omniverse Isaac Gym environment environment
+        :type env: Any supported Omniverse Isaac Gym environment
         """
         super().__init__(env)
 
@@ -262,6 +262,53 @@ class OmniverseIsaacGymWrapper(Wrapper):
         self._env.close()
 
 
+class IsaacOrbitWrapper(Wrapper):
+    def __init__(self, env: Any) -> None:
+        """Isaac Orbit environment wrapper
+
+        :param env: The environment to wrap
+        :type env: Any supported Isaac Orbit environment
+        """
+        super().__init__(env)
+
+        self._reset_once = True
+        self._obs_dict = None
+
+    def step(self, actions: torch.Tensor) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor, torch.Tensor, Any]:
+        """Perform a step in the environment
+
+        :param actions: The actions to perform
+        :type actions: torch.Tensor
+
+        :return: Observation, reward, terminated, truncated, info
+        :rtype: tuple of torch.Tensor and any other info
+        """
+        self._obs_dict, reward, terminated, info = self._env.step(actions)
+        truncated = torch.zeros_like(terminated)
+        return self._obs_dict["policy"], reward.view(-1, 1), terminated.view(-1, 1), truncated.view(-1, 1), info
+
+    def reset(self) -> Tuple[torch.Tensor, Any]:
+        """Reset the environment
+
+        :return: Observation, info
+        :rtype: torch.Tensor and any other info
+        """
+        if self._reset_once:
+            self._obs_dict = self._env.reset()
+            self._reset_once = False
+        return self._obs_dict["policy"], {}
+
+    def render(self, *args, **kwargs) -> None:
+        """Render the environment
+        """
+        pass
+
+    def close(self) -> None:
+        """Close the environment
+        """
+        self._env.close()
+
+
 class GymWrapper(Wrapper):
     def __init__(self, env: Any) -> None:
         """OpenAI Gym environment wrapper
@@ -925,6 +972,8 @@ def wrap_env(env: Any, wrapper: str = "auto", verbose: bool = True) -> Wrapper:
                     +--------------------+-------------------------+
                     |Omniverse Isaac Gym |``"omniverse-isaacgym"`` |
                     +--------------------+-------------------------+
+                    |Isaac Sim (orbit)   |``"isaac-orbit"``        |
+                    +--------------------+-------------------------+
     :type wrapper: str, optional
     :param verbose: Whether to print the wrapper type (default: True)
     :type verbose: bool, optional
@@ -945,6 +994,12 @@ def wrap_env(env: Any, wrapper: str = "auto", verbose: bool = True) -> Wrapper:
                 logger.info("Environment wrapper: Omniverse Isaac Gym")
             return OmniverseIsaacGymWrapper(env)
         elif isinstance(env, gym.core.Env) or isinstance(env, gym.core.Wrapper):
+            # isaac-orbit
+            if hasattr(env, "sim") and hasattr(env, "env_ns"):
+                if verbose:
+                    logger.info("Environment wrapper: Isaac Orbit")
+                return IsaacOrbitWrapper(env)
+            # gym
             if verbose:
                 logger.info("Environment wrapper: Gym")
             return GymWrapper(env)
@@ -999,5 +1054,9 @@ def wrap_env(env: Any, wrapper: str = "auto", verbose: bool = True) -> Wrapper:
         if verbose:
             logger.info("Environment wrapper: Omniverse Isaac Gym")
         return OmniverseIsaacGymWrapper(env)
+    elif wrapper == "isaac-orbit":
+        if verbose:
+            logger.info("Environment wrapper: Isaac Orbit")
+        return IsaacOrbitWrapper(env)
     else:
         raise ValueError("Unknown {} wrapper type".format(wrapper))
