@@ -1,6 +1,7 @@
-from typing import Optional, Sequence
+from typing import Union, Mapping, Tuple, Any
 
 import gym
+import gymnasium
 
 import torch
 
@@ -20,7 +21,7 @@ class DeterministicMixin:
             >>> import torch
             >>> import torch.nn as nn
             >>> from skrl.models.torch import Model, DeterministicMixin
-            >>> 
+            >>>
             >>> class Value(DeterministicMixin, Model):
             ...     def __init__(self, observation_space, action_space, device="cuda:0", clip_actions=False):
             ...         Model.__init__(self, observation_space, action_space, device)
@@ -32,13 +33,13 @@ class DeterministicMixin:
             ...                                  nn.ELU(),
             ...                                  nn.Linear(32, 1))
             ...
-            ...     def compute(self, states, taken_actions, role):
-            ...         return self.net(states)
+            ...     def compute(self, inputs, role):
+            ...         return self.net(inputs["states"]), {}
             ...
             >>> # given an observation_space: gym.spaces.Box with shape (60,)
             >>> # and an action_space: gym.spaces.Box with shape (8,)
             >>> model = Value(observation_space, action_space)
-            >>> 
+            >>>
             >>> print(model)
             Value(
               (net): Sequential(
@@ -52,7 +53,8 @@ class DeterministicMixin:
         """
         if not hasattr(self, "_d_clip_actions"):
             self._d_clip_actions = {}
-        self._d_clip_actions[role] = clip_actions and issubclass(type(self.action_space), gym.Space)
+        self._d_clip_actions[role] = clip_actions and (issubclass(type(self.action_space), gym.Space) or \
+            issubclass(type(self.action_space), gymnasium.Space))
 
         if self._d_clip_actions[role]:
             self.clip_actions_min = torch.tensor(self.action_space.low, device=self.device, dtype=torch.float32)
@@ -60,42 +62,39 @@ class DeterministicMixin:
 
             # backward compatibility: torch < 1.9 clamp method does not support tensors
             self._backward_compatibility = tuple(map(int, (torch.__version__.split(".")[:2]))) < (1, 9)
-        
-    def act(self, 
-            states: torch.Tensor, 
-            taken_actions: Optional[torch.Tensor] = None, 
-            role: str = "") -> Sequence[torch.Tensor]:
+
+    def act(self,
+            inputs: Mapping[str, Union[torch.Tensor, Any]],
+            role: str = "") -> Tuple[torch.Tensor, Union[torch.Tensor, None], Mapping[str, Union[torch.Tensor, Any]]]:
         """Act deterministically in response to the state of the environment
 
-        :param states: Observation/state of the environment used to make the decision
-        :type states: torch.Tensor
-        :param taken_actions: Actions taken by a policy to the given states (default: ``None``).
-                              The use of these actions only makes sense in critical models, e.g.
-        :type taken_actions: torch.Tensor, optional
+        :param inputs: Model inputs. The most common keys are:
+
+                       - ``"states"``: state of the environment used to make the decision
+                       - ``"taken_actions"``: actions taken by the policy for the given states
+        :type inputs: dict where the values are typically torch.Tensor
         :param role: Role play by the model (default: ``""``)
         :type role: str, optional
 
-        :return: Action to be taken by the agent given the state of the environment.
-                 The sequence's components are the computed actions and None for the last two components
-        :rtype: sequence of torch.Tensor
+        :return: Model output. The first component is the action to be taken by the agent.
+                 The second component is ``None``. The third component is a dictionary containing extra output values
+        :rtype: tuple of torch.Tensor, torch.Tensor or None, and dictionary
 
         Example::
 
             >>> # given a batch of sample states with shape (4096, 60)
-            >>> output = model.act(states)
-            >>> print(output[0].shape, output[1], output[2])
-            torch.Size([4096, 1]) None None
+            >>> actions, _, outputs = model.act({"states": states})
+            >>> print(actions.shape, outputs)
+            torch.Size([4096, 1]) {}
         """
         # map from observations/states to actions
-        actions = self.compute(states.to(self.device), 
-                               taken_actions.to(self.device) if taken_actions is not None else taken_actions, role)
+        actions, outputs = self.compute(inputs, role)
 
-        # clip actions 
+        # clip actions
         if self._d_clip_actions[role] if role in self._d_clip_actions else self._d_clip_actions[""]:
             if self._backward_compatibility:
                 actions = torch.max(torch.min(actions, self.clip_actions_max), self.clip_actions_min)
             else:
                 actions = torch.clamp(actions, min=self.clip_actions_min, max=self.clip_actions_max)
 
-        return actions, None, None
-        
\ No newline at end of file
+        return actions, None, outputs
